import struct
from collections import OrderedDict
from protobuf_to_dict import protobuf_to_dict

try:
    import manifest_pb2
except ImportError:
    print '\nINSTALL: python-protobuf, protobuf-compiler'
    print 'RUN: protoc -I=. --python_out=. ./manifest.proto\n'
    raise

MSG_PAYLOAD = 0x71F617D0
MSG_METADATA = 0x1F4812BE
MSG_SIGNATURE = 0x1B81B817
MSG_EOF = 0x32C415AB

MSG_NAMES = {
    MSG_PAYLOAD: 'payload',
    MSG_METADATA: 'metadata',
    MSG_SIGNATURE: 'signature'
}


class Manifest:

    MessageClass = {
        MSG_PAYLOAD: manifest_pb2.Payload,
        MSG_METADATA: manifest_pb2.Metadata,
        MSG_SIGNATURE: manifest_pb2.Signature
    }

    def __init__(self, content):
        self._content = content
        self._offset = 0
        self.dict = OrderedDict()

        while True:
            msg_id = self.read_struct('I')
            if msg_id == MSG_EOF:
                break

            msg_size = self.read_struct('I')
            msg_data = self.read_struct(str(msg_size) + 's')

            message = self.MessageClass[msg_id]()
            message.ParseFromString(msg_data)
            self.dict[MSG_NAMES[msg_id]] = protobuf_to_dict(message)

    def read_struct(self, fmt):
        result = struct.unpack_from(fmt, self._content, self._offset)
        self._offset += struct.calcsize(fmt)

        # A hack to return the value itself instead of a tuple of one element.
        if len(result) == 1:
            return result[0]
        else:
            return result
